#!/usr/bin/env python
# Written by: DGC

# python imports
import os

# local imports

resource_directory = os.path.dirname(__file__)

#==============================================================================
def find_resource_file(file_name):
    """ 
    file_name should be a relative path under Resources, this will return the 
    absolute path of the file.
    """
    path = os.path.join(resource_directory, file_name)
    assert os.path.isfile(path)
    return path

#==============================================================================
def find_image_file(file_name):
    """ 
    file_name should be a relative path under Resources, this will return the 
    absolute path of the file.
    """
    return find_resource_file(os.path.join("Images", file_name))

#==============================================================================
def resource_file_exists(file_name):
    """ Returns if file_name exists """
    path = os.path.join(resource_directory, file_name)
    return os.path.isfile(path)



#!/usr/bin/env python
# Written by: DGC

# python imports

# local imports
import qtGUI

def main():
    application = qtGUI.MainApp()
    application.MainLoop()

#==============================================================================
if (__name__ == "__main__"):
    main()



#!/usr/bin/env python
# Written by: DGC

# python imports
import unittest

# local imports
import DiceRollers

#==============================================================================
class utest_DiceRollers(unittest.TestCase):

    def test_range(self):
        for i in range(3, 21):
            die = DiceRollers.Dice(i)
            for j in range(100000):
                num = die.roll()
                assert 1 <= num, "num is under 1"
                assert num <= i, "num is more than it should be"
                assert type(num) == int
            self.assertTrue(True, "Worked for a d" + str(i))
    
    def test_expected(self):
        d3 = DiceRollers.Dice(3)
        self.assertEqual(d3.expected, 2, "D3 expected value is correct.")
    
        d4 = DiceRollers.Dice(4)
        self.assertEqual(d4.expected, 2.5, "D4 expected value is correct.")
    
        d5 = DiceRollers.Dice(6)
        self.assertEqual(d5.expected, 3.5, "D6 expected value is correct.")
    
        d8 = DiceRollers.Dice(8)
        self.assertEqual(d8.expected, 4.5, "D8 expected value is correct.")
    
        d10 = DiceRollers.Dice(10)
        self.assertEqual(d10.expected, 5.5, "D10 expected value is correct.")
    
        d12 = DiceRollers.Dice(12)
        self.assertEqual(d12.expected, 6.5, "D12 expected value is correct.")
    
        d20 = DiceRollers.Dice(20)
        self.assertEqual(d20.expected, 10.5, "D20 expected value is correct.")
    
        d100 = DiceRollers.Dice(100)
        self.assertEqual(
            d100.expected,
            50.5,
            "D100 expected value is correct."
            )
    
        pool = DiceRollers.DicePool()
        self.assertEqual(pool.expected, 0, "Empty pool has expected value 0.")
    
        pool.add_dice("D3")
        self.assertEqual(pool.expected, 2, "Pool [D3] has expected value 2.")
    
        pool.add_dice("D3")
        self.assertEqual(
            pool.expected, 
            4,
            "Pool [D3, D3] has expected value 4."
            )
    
        pool = DiceRollers.DicePool()
        pool.add_dice("D6")
        self.assertEqual(
            pool.expected,
            3.5,
            "Pool [D6] has expected value 3.5."
            )
    
        pool.add_dice("D6")
        self.assertEqual(
            pool.expected, 
            7,
            "Pool [D6, D6] has expected value 7."
            )
    
        pool.add_dice("D3")
        pool.add_dice("D8")
        pool.add_dice("D10")
        pool.add_dice("D12")
        pool.add_dice("D20")
        pool.add_dice("D100")
        
        message = "Pool [D3, D6, D6, D8, D10, D12, D20, D100] "
        message += "has expected value 86.5."
        self.assertEqual(pool.expected, 86.5, message)

#==============================================================================
if (__name__ == "__main__"):
    unittest.main(verbosity=2)



#!/usr/bin/env python
# Written by: DGC

# python imports
from PyQt4 import QtGui, QtCore

import sys
import time
import os

# local imports
import DiceRollers
import PdfViewer
import ProbabilityDensity
import ProgramUtils
import Resources.finder

#==============================================================================
class MainApp(QtGui.QApplication):

    def __init__(self):
        super(MainApp, self).__init__(sys.argv)
        self.window = MainWindow()

    def MainLoop(self):
        self.window.show()
        sys.exit(self.exec_())

#==============================================================================
class MainWindow(QtGui.QMainWindow):

    def __init__(self):
        super(MainWindow, self).__init__()

        program = ProgramUtils.Program()

        self.setWindowTitle(
            program.info["name"] + " " + program.info["version"]
            )

        self.setWindowIcon(
            QtGui.QIcon(Resources.finder.find_image_file(program.info["icon"]))
            )
        self.center()

        self.dice_pool = DiceRollers.DicePool()

        self.file_dialog_dir = "."
        self.initialise_UI()

    def center(self):
        """ Centre the window in the screen. """
        qr = self.frameGeometry()
        cp = QtGui.QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

    def initialise_UI(self):
        """ Make all the widgets, toolbars and status bar. """
        self.dice_selecter = DiceSelecterDock()
        QtCore.QObject.connect(
            self.dice_selecter,
            QtCore.SIGNAL("dice_added"),
            self.dice_pool.add_dice
            )

        dice_pool_viewer = DicePoolViewer(self.dice_pool)

        QtCore.QObject.connect(
            self.dice_selecter,
            QtCore.SIGNAL("dice_added"),
            dice_pool_viewer.update_table
            )

        QtCore.QObject.connect(
            dice_pool_viewer,
            QtCore.SIGNAL("show_probability"),
            self.open_probability_window
            )
        self.setCentralWidget(dice_pool_viewer)

        dice_history = DiceHistoryDock()
        QtCore.QObject.connect(
            dice_pool_viewer,
            QtCore.SIGNAL("dice_rolled"),
            dice_history.update
            )
        QtCore.QObject.connect(
            dice_pool_viewer,
            QtCore.SIGNAL("reset"),
            dice_history.reset
            )

        # set the corner areas for docking
        self.setCorner(
            QtCore.Qt.TopLeftCorner,
            QtCore.Qt.LeftDockWidgetArea
            )
        self.setCorner(
            QtCore.Qt.BottomLeftCorner,
            QtCore.Qt.LeftDockWidgetArea
            )
        self.setCorner(
            QtCore.Qt.BottomRightCorner,
            QtCore.Qt.RightDockWidgetArea
            )
        self.setCorner(
            QtCore.Qt.TopRightCorner,
            QtCore.Qt.RightDockWidgetArea
            )

        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.dice_selecter)
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, dice_history)
        self.make_menu_bar()

    def make_menu_bar(self):
        pdf_menu = self.menuBar().addMenu("&PDF")
        open_local_pdf = pdf_menu.addAction("&Open Local PDF")
        open_local_pdf.setIcon(
            QtGui.QIcon(Resources.finder.find_image_file("Pdf.png"))
            )
        QtCore.QObject.connect(
            open_local_pdf,
            QtCore.SIGNAL("triggered()"),
            self.open_local_pdf
            )

        open_remote_pdf = pdf_menu.addAction("PDF From &URL")
        open_remote_pdf.setIcon(
            QtGui.QIcon(Resources.finder.find_image_file("Pdf.png"))
            )
        QtCore.QObject.connect(
            open_remote_pdf,
            QtCore.SIGNAL("triggered()"),
            self.open_remote_pdf
            )

        tool_menu = self.menuBar().addMenu("&Tools")
        add_dice = tool_menu.addAction("&New Dice")
        # use a D6 icon
        add_dice.setIcon(
            QtGui.QIcon(Resources.finder.find_image_file("D6.png"))
            )
        QtCore.QObject.connect(
            add_dice,
            QtCore.SIGNAL("triggered()"),
            self.raise_dice_creator
            )

        restore_defaults = tool_menu.addAction(
            "&Restore Default Configuration"
            )
        restore_defaults.setIcon(
            QtGui.QIcon(Resources.finder.find_image_file("Restore.png"))
            )
        QtCore.QObject.connect(
            restore_defaults,
            QtCore.SIGNAL("triggered()"),
            self.restore_default_config
            )
        context_menu = self.createPopupMenu()
        context_menu.setTitle("&View")
        self.menuBar().addMenu(context_menu)

    def restore_default_config(self):
        message_box = QtGui.QMessageBox()
        message_box.setText("Reset the configuration?")
        message_box.setWindowTitle("Configuration Reset")

        message_box.setWindowIcon(
            QtGui.QIcon(Resources.finder.find_image_file(program.info["icon"]))
            )

        current_session = QtGui.QPushButton("Just for this session")
        message_box.addButton(
            current_session,
            QtGui.QMessageBox.AcceptRole
            )
        always = QtGui.QPushButton("Permanently")
        message_box.addButton(always, QtGui.QMessageBox.DestructiveRole)

        no = QtGui.QPushButton("Cancel")
        message_box.addButton(no, QtGui.QMessageBox.RejectRole)
        
        message_box.exec_()
        reset = False
        if (message_box.clickedButton() == current_session):
            reset = True
        elif (message_box.clickedButton() == always):
            reset = True
            custom_xml = Resources.finder.find_resource_file("Custom.xml")
            if (os.path.isfile(custom_xml)):
                os.remove(custom_xml)
        if (reset):
            program = ProgramUtils.Program()
            program.restore_defaults()
            self.dice_selecter.dice_selecter.set_library(program.dice_library)

    def raise_dice_creator(self):
        creator = DiceCreator()
        if (creator.exec_()):
            dice = creator.dice
            # add it to the library, and Config.xml
            library = ProgramUtils.Program().dice_library
            library.add_dice(dice)
            # reset the dice selector
            self.dice_selecter.dice_selecter.set_library(library)

    def open_local_pdf(self):
        dialog = QtGui.QFileDialog()
        dialog.setFileMode(QtGui.QFileDialog.ExistingFile)
        file_extensions = "PDF (*.pdf)"
        file_name = dialog.getOpenFileName(
            self,
            "PDF",
            self.file_dialog_dir,
            file_extensions
            )
        if (file_name):
            # this means the dialog was not cancelled
            # save the directory
            self.file_dialog_dir = os.path.dirname(str(file_name))
            # make QUrl from the name
            url = QtCore.QUrl.fromLocalFile(file_name)
            # view the pdf
            self.__view_pdf(url)

    def open_remote_pdf(self):
        # get a Url
        dialog = QtGui.QInputDialog()
        url_as_text = dialog.getText(
            self,
            "PDF From URL",
            "Enter URL"
            )
        # this returns a tuple of the form (text entered, ok clicked)
        if (url_as_text[1]):
            # if ok was clicked
            url = QtCore.QUrl(url_as_text[0])
            self.__view_pdf(url)

    def __view_pdf(self, url):
        # open the Url in the PdfViewer
        viewer = PdfViewer.ViewerDock()
        viewer.set_url(url)
        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, viewer)
        viewer.setFloating(True)

    def open_probability_window(self):
        self.probability_window = ProbabilityDensityDock(self.dice_pool)
        self.addDockWidget(
            QtCore.Qt.LeftDockWidgetArea, 
            self.probability_window
            )
        self.probability_window.setFloating(True)

#==============================================================================
class DiceSelecterDock(QtGui.QDockWidget):

    def __init__(self):
        super(DiceSelecterDock, self).__init__("Dice Selecter")
        self.dice_selecter = DiceSelecter()
        QtCore.QObject.connect(
            self.dice_selecter,
            QtCore.SIGNAL("dice_added"),
            self.emit_signal
            )
        self.setWidget(self.dice_selecter)

    def emit_signal(self, dice_name):
        self.emit(QtCore.SIGNAL("dice_added"), dice_name)

#==============================================================================
class DiceSelecter(QtGui.QWidget):

    def __init__(self):
        super(DiceSelecter, self).__init__()
        self.layout = QtGui.QVBoxLayout()

        library = ProgramUtils.Program().dice_library
        self.set_library(library)

        # make the window wider
        self.setMinimumWidth(120)

    def set_library(self, library):
        # clear the current library first
        for i in range(self.layout.count()):
           self.layout.itemAt(i).widget().close()
        # sort the names on the numbers of sides of the library item.
        names = sorted(
            library.keys(),
            lambda x,y: cmp(library[x].sides, library[y].sides)
            )
        for name in names:
            self.append_dice(library[name])

    def append_dice(self, dice):
        # make the button
        button = QtGui.QToolButton()

        # if there is a label, set it as the tool tip
        if (not dice.label is None):
            button.setToolTip(dice.label)

        # if there is an image set it as the icon, otherwise set the label as
        # the text
        if (not dice.image is None):
            button.setIcon(QtGui.QIcon(dice.image))
        else:
            button.setText(dice.label)

        button.clicked.connect(lambda event: self.append_event(dice))

        # make the buttons and button icon larger
        button.setMinimumSize(QtCore.QSize(60, 60))
        button.setIconSize(QtCore.QSize(60, 60))

        self.layout.addWidget(button, 0, QtCore.Qt.AlignHCenter)
        self.setLayout(self.layout)

    def append_event(self, dice):
        self.emit(QtCore.SIGNAL("dice_added"), dice.label)

#==============================================================================
class DicePoolViewer(QtGui.QWidget):

    def __init__(self, dice_pool):
        super(DicePoolViewer, self).__init__()
        self.dice_pool = dice_pool

        self.table = QtGui.QTableWidget(0, 4, self)
        self.table.setMinimumSize(QtCore.QSize(400, 200))
        self.initialise_table()

        roll_button = QtGui.QPushButton("&Roll")
        hotkey = QtGui.QShortcut("Enter", self)
        QtCore.QObject.connect(
            hotkey,
            QtCore.SIGNAL("activated()"),
            self.roll
            )
        roll_button.clicked.connect(self.roll)

        reset_button = QtGui.QPushButton("Reset")
        reset_button.clicked.connect(self.reset)

        vertical_layout = QtGui.QVBoxLayout()
        vertical_layout.addWidget(self.table, 2, QtCore.Qt.AlignHCenter)
        
        self.probablity = QtGui.QLabel()
        self.update_probablity()

        distribution_button = QtGui.QPushButton("Show Probabilities")
        distribution_button.clicked.connect(self.show_probabilities)

        middle_layout = QtGui.QHBoxLayout()
        middle_layout.addWidget(self.probablity)
        middle_layout.addWidget(distribution_button)
        
        vertical_layout.addLayout(middle_layout)

        button_layout = QtGui.QHBoxLayout()
        button_layout.addWidget(roll_button)
        button_layout.addWidget(reset_button)

        vertical_layout.addLayout(button_layout, 0)

        self.setLayout(vertical_layout)

    def initialise_table(self):
        self.table.setRowCount(0)
        self.table.setColumnCount(4)
        self.table.setHorizontalHeaderLabels(
            ["Dice", "Number", "Add", "Remove"]
            )
        self.table.verticalHeader().hide()

        self.table.horizontalHeader().setStretchLastSection(False)

    def update_table(self):
        self.table.clear()
        self.initialise_table()

        for dice in self.dice_pool:
            # add the dice label/image to the table
            self.table.setRowCount(self.table.rowCount() + 1)

            label = None
            if (not dice.image is None):
                label = QtGui.QTableWidgetItem(
                    QtGui.QIcon(dice.image),
                    dice.label
                    )
            else:
                label = QtGui.QTableWidgetItem(dice.label)
            label.setFlags(QtCore.Qt.ItemIsEnabled)
            row_number = self.table.rowCount() - 1
            self.table.setItem(row_number, 0, label)

            # add the number of dice in the pool
            number_label = QtGui.QTableWidgetItem(str(self.dice_pool[dice]))
            number_label.setFlags(QtCore.Qt.ItemIsEnabled)
            number_label.setTextAlignment(QtCore.Qt.AlignCenter)
            self.table.setItem(row_number, 1, number_label)

            # put add button in table
            add = AddButton(dice.label)
            QtCore.QObject.connect(
                add,
                QtCore.SIGNAL("add_dice"),
                self.add_dice
                )
            self.table.setCellWidget(row_number, 2, add)

            # add remove button to table
            remove = RemoveButton(dice.label)
            QtCore.QObject.connect(
                remove,
                QtCore.SIGNAL("remove_dice"),
                self.remove_dice
                )
            self.table.setCellWidget(row_number, 3, remove)

            self.update_probablity()

    def add_dice(self, dice_name):
        self.dice_pool.add_dice(dice_name)
        self.update_table()

    def remove_dice(self, dice_name):
        self.dice_pool.remove_dice(dice_name)
        self.update_table()

    def update_probablity(self):
        label = "Expected Value: " + str(self.dice_pool.expected) + "\n"
        label += "Minimum: " + str(self.dice_pool.minimum) + "\n"
        label += "Maximum: " + str(self.dice_pool.maximum)
        self.probablity.setText(label)

    def roll(self):
        value = self.dice_pool.roll()
        self.emit(QtCore.SIGNAL("dice_rolled"), value)

    def reset(self):
        self.dice_pool.clear()
        self.table.clear()
        self.initialise_table()
        self.update_probablity()
        self.emit(QtCore.SIGNAL("reset"))

    def show_probabilities(self):
        self.emit(QtCore.SIGNAL("show_probability"))

#==============================================================================
class DiceHistoryDock(QtGui.QDockWidget):

    def __init__(self):
        super(DiceHistoryDock, self).__init__("Dice Rolls")
        self.setFeatures(
            QtGui.QDockWidget.DockWidgetFloatable |
            QtGui.QDockWidget.DockWidgetMovable
            )
        self.history = DiceHistory()
        self.setWidget(self.history)

    def update(self, summery):
        self.history.update(summery)

    def reset(self):
        self.history.reset()

#==============================================================================
class DiceHistory(QtGui.QWidget):

    def __init__(self):
        super(DiceHistory, self).__init__()

        self.output = QtGui.QTextEdit()
        self.output.setReadOnly(True)

        layout = QtGui.QHBoxLayout()
        layout.addWidget(self.output)
        self.setLayout(layout)

    def update(self, summery):
        # return the cursor the the begining
        self.output.setTextCursor(QtGui.QTextCursor(self.output.document()))
        self.output.insertHtml(
            "<p>" +
            time.strftime("%d-%m %H:%M", time.localtime()) +
            " - " +
            "<FONT COLOR=\"#FF0000\">" +
            str(summery) +
            "</FONT>" +
            "<br><br></p>"
            )

    def reset(self):
        # return the cursor the the begining
        self.output.setTextCursor(QtGui.QTextCursor(self.output.document()))
        self.output.insertHtml(
            "<p>" +
            time.strftime("%d-%m %H:%M", time.localtime()) +
            " - " +
            "<FONT COLOR=\"#FF0000\">" +
            "Reset" +
            "</FONT>" +
            "<br><br></p>"
            )

#==============================================================================
class DiceCreator(QtGui.QDialog):

    def __init__(self):
        super(DiceCreator, self).__init__()
        self.setWindowTitle("Dice Creator")

        self.setWindowIcon(
            QtGui.QIcon(Resources.finder.find_image_file(program.info["icon"]))
            )
        self.setModal(True)

        self.initialise_ui()
        self.dice = None
        self.image = None
        self.file_dialog_dir = "."

    def initialise_ui(self):
        name = QtGui.QLabel("Dice name: ")
        self.name_input = QtGui.QLineEdit()

        sides = QtGui.QLabel("Number of Sides: ")
        self.sides_input = QtGui.QLineEdit()

        self.image_button = QtGui.QPushButton("Select Image")
        self.image_button.clicked.connect(self.find_image)

        ok_button = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok)
        ok_button.clicked.connect(self.accept)

        cancel_button = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Cancel)
        cancel_button.clicked.connect(self.reject)

        layout = QtGui.QGridLayout()
        layout.addWidget(name, 0, 0)
        layout.addWidget(self.name_input, 0, 1)

        layout.addWidget(sides, 1, 0)
        layout.addWidget(self.sides_input, 1, 1)

        layout.addWidget(self.image_button, 2, 0, 1, 2)

        layout.addWidget(ok_button, 3, 0)
        layout.addWidget(cancel_button, 3, 1)

        self.setLayout(layout)

    def accept(self):
        if (self.validate()):
            self.dice = ProgramUtils.DiceLibraryEntry(
                str(self.name_input.text()),
                int(self.sides_input.text()),
                self.image
                )
            super(DiceCreator, self).accept()

    def validate(self):
        ok = True
        error = ""
        if (self.name_input.text() == ""):
            error = "Undefined dice name."
            ok = False
        else:
            if (self.sides_input.text() == ""):
                error =  "Undefined number of sides."
                ok = False
            else:
                try:
                    sides = int(self.sides_input.text())
                    if (sides <= 0):
                        error = "Dice cannot have fewer than 0 sides."
                        ok = False
                except ValueError:
                    error = "Dice cannot have \""
                    error += str(self.sides_input.text())
                    error += "\" sides."
                    ok = False
        if (not ok):
            error_dialog = QtGui.QErrorMessage(self)
            error_dialog.showMessage(error)
            error_dialog.exec_()
        return ok

    def find_image(self):
        dialog = QtGui.QFileDialog()
        dialog.setFileMode(QtGui.QFileDialog.ExistingFile)
        file_extensions = "Images (*.bmp *.gif *.jpg *.png);;All files (*)"
        file_name = dialog.getOpenFileName(
            self,
            "Find Image",
            self.file_dialog_dir,
            file_extensions
            )
        if (file_name):
            # this means the dialog was not cancelled
            # save the directory
            self.file_dialog_dir = os.path.dirname(str(file_name))
            self.image = str(file_name)
            self.image_button.setText(os.path.basename(self.image))

#==============================================================================
class RemoveButton(QtGui.QToolButton):

    def __init__(self, dice_name):
        super(RemoveButton, self).__init__()
        self.dice_name = dice_name

        self.setMaximumSize(QtCore.QSize(40, 40))

        self.setIcon(
            QtGui.QIcon(Resources.finder.find_image_file("Remove.png"))
            )
        self.setToolTip("Remove 1 " + dice_name)
        self.clicked.connect(self.send_remove_call)

    def send_remove_call(self):
        self.emit(QtCore.SIGNAL("remove_dice"), self.dice_name)

#==============================================================================
class AddButton(QtGui.QToolButton):

    def __init__(self, dice_name):
        super(AddButton, self).__init__()
        self.dice_name = dice_name

        self.setMaximumSize(QtCore.QSize(40, 40))

        self.setIcon(
            QtGui.QIcon(Resources.finder.find_image_file("Add.png"))
            )
        self.setToolTip("Add 1 " + dice_name)
        self.clicked.connect(self.send_add_call)

    def send_add_call(self):
        self.emit(QtCore.SIGNAL("add_dice"), self.dice_name)

#==============================================================================
class ProbabilityDensityDock(QtGui.QDockWidget):

    def __init__(self, dice_pool):
        super(ProbabilityDensityDock, self).__init__("Probability Density")
        self.density_widget = ProbabilityDensityWidget(dice_pool)
        self.setWidget(self.density_widget)        

#==============================================================================
class ProbabilityDensityWidget(QtGui.QWidget):

    def __init__(self, dice_pool):
        super(ProbabilityDensityWidget, self).__init__()
        self.dice_pool = dice_pool
        self.density = ProbabilityDensity.ProbabilityDensity(self.dice_pool)
        self.setMinimumSize(QtCore.QSize(550, 350))
        
    def update(self):
        self.density.make_density()
        self.repaint()
        
    def paintEvent(self, e):
        # if the probability is too much to calculate
        if (not self.density.density is None):
            painter = QtGui.QPainter()
            painter.begin(self)
            self.draw_graph(painter)
            painter.end()
        
    def draw_graph(self, painter):
        pen = QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.SolidLine)

        painter.setPen(pen)
        painter.setFont(QtGui.QFont("decorative", 8))
        
        # get the maximum density
        highest = max(
            self.density.density, 
            key = lambda x: self.density.density[x]
            )
        highest = self.density.density[highest]
        # the scale in y, fit to 240
        scale = 240 / highest
        if (scale < 1):
            scale = 1
        # the number of probabilities
        probability_number = len(self.density.density.keys())
        # the space between numbers in x, fit to 480
        space = 480 / probability_number
        # the value where the x axis is. highest goes down one as the graph 
        # starts from 0
        base = space + ((highest) * scale)
        # draw the chart
        for i, value in enumerate(self.density.density.keys()):
            number = self.density.density[value]
            painter.drawLine(
                space * (i + 1),
                base, 
                space * (i + 1),
                base - (scale * number)
                )
            painter.drawText(space * (i + 1), base + 20, str(value))

        # draw y axis
        painter.drawLine(space / 2, space, space / 2, base)
        # draw x axis
        painter.drawLine(
            space / 2,
            base, 
            space * (probability_number + 1.5), 
            base
            )

    def draw_scale(self):
        total = self.density.possibilities
        for i in range(1, highest):
            painter.drawLine(
                space / 2,
                base - (scale * i),
                space, 
                base - (scale * i)
                )
            painter.drawText(space, base - (scale * i), str(i / float(total)))



#!/usr/bin/env python
# Written by: DGC

# python imports
import random

# local imports
import ProgramUtils

#==============================================================================
class Dice(object):
    
    def __init__(self, sides):
        self.sides = sides
        self.expected = sum(range(1, self.sides + 1)) / float(self.sides)

    def roll(self):
        random.seed()
        return random.randrange(1, self.sides + 1)
        
#==============================================================================
class DicePool(object):
    
    def __init__(self):
        self.dice = dict()

    @property
    def pool(self):
        """ Return a list with one DiceEntry for each die in the pool. """
        return self.dice.keys()
            
    @property
    def expected(self):
        expected = 0
        for die in self:
            expected += (Dice(die.sides).expected * self.dice[die])
        return expected

    @property
    def maximum(self):
        maximum = 0
        for dice in self:
            maximum += (dice.sides * self[dice])
        return maximum

    @property
    def minimum(self):
        minimum = 0
        for dice in self:
            minimum += self[dice]
        return minimum

    def roll(self):
        total = 0
        for die in self.pool:
            for i in range(self[die]):
                total += Dice(die.sides).roll()
        return total

    def add_dice(self, name):
        dice = ProgramUtils.Program().dice_library[name]
        # add one to the dictionary if it exists, otherwise make it equal to 1.
        if (dice in self.dice):
            self.dice[dice] += 1
        else:
            self.dice[dice] = 1

    def remove_dice(self, name):
        dice = ProgramUtils.Program().dice_library[name]
        # assert there is a dice
        assert dice in self.dice, "No " + name + " in dictionary."
        # remove from dictionary if it is 1, otherwise decrement it
        if (self.dice[dice] == 1):
            del self.dice[dice]
        else:
            self.dice[dice] -= 1

    def clear(self):
        self.dice = dict()

    #--------------------------------------------------------------------------
    # Magic methods to implement this as a list
    #--------------------------------------------------------------------------

    def __len__(self):
        """ Returns the number of dice in the pool. """
        return len(self.pool)

    def __getitem__(self, dice):
        return self.dice[dice]

    def __iter__(self):
        """ Iterate over the dice in the pool. """
        return iter(self.pool)



#!/usr/bin/env python
# Written by: DGC

# python imports
import itertools
# local imports

#==============================================================================
class ProbabilityDensity(object):
    
    def __init__(self, dice_pool):
        self.dice_pool = dice_pool
        self.make_density()

    def __getitem__(self, index):
        ret = None
        if index in self.density:
            ret = self.density[index] / float(self.possibilities)
        else:
            ret = 0
        return ret

    def make_density(self):
        self.find_possibilities()
        if (self.possibilities < 1000000):
            possible = []
            for dice in self.dice_pool.pool:
                # do this once for each dice of that type in the pool
                for i in range(self.dice_pool[dice]):
                    current = []
                    for j in range(1, dice.sides + 1):
                        current.append(j)
                    possible.append(current)
            
            self.density = dict()
            for possibility in list(itertools.product(*possible)):
                score = 0
                for result in possibility:
                    score += result
                if score in self.density:
                    self.density[score] += 1
                else:
                    self.density[score] = 1
        else:
            # too many to calculate
            self.density = None
            
    def find_possibilities(self):
        """ Finds the number of possible outcomes in the density function. """
        if (len(self.dice_pool) == 0):
            self.possibilities = 0
        else:
            self.possibilities = 1
            for dice in self.dice_pool:
                self.possibilities *= (dice.sides ** self.dice_pool[dice])



#!/usr/bin/env python
# Written by: DGC

# python imports
import os
import xml.etree.ElementTree as ET

# local imports
import Resources.finder

#==============================================================================
class Program(object):
    """ Class to store all the data about the program, version etc. """
    # this is now a singleton so that data reading gets done once.

    __instance = None

    def __new__(new_singleton):
        """Override the __new__ method so that it is a singleton."""
        if not new_singleton.__instance:
            new_singleton.__instance = object.__new__(new_singleton)
            new_singleton.__instance.init()
        return new_singleton.__instance

    def init(self):
        # cache these
        self.__info = None
        self.__dice_library = None

    @property
    def info(self):
        if (self.__info is None):
            self.__info = dict()
    
            # read the information from Config.xml
            tree = ET.parse(Resources.finder.find_resource_file("Config.xml"))
            node = tree.find("ProgramInformation")
            for item in node:
                self.__info[item.tag] = item.text
        return self.__info
    
    @property
    def dice_library(self):
        if (self.__dice_library is None):
            self.__dice_library = DiceLibrary()
        return self.__dice_library

    def restore_defaults(self):
        self.__dice_library = DiceLibrary(custom = False)

#==============================================================================
class DiceLibrary(dict):

    def __init__(self, custom = True):
        super(DiceLibrary, self).__init__()
        # find Custom.xml over the "installed" version Config.xml
        exists = Resources.finder.resource_file_exists("Custom.xml")
        if (exists and custom):
            xml = Resources.finder.find_resource_file("Custom.xml")
        else:
            xml = Resources.finder.find_resource_file("Config.xml")
        
        # read the xml file to get the dice
        tree = ET.parse(xml)
        if (float(tree.getroot().get("version")) == 1):
            dice_pools = tree.findall("DicePool")
            for pool in dice_pools:
                dice = pool.findall("Dice")
                for die in dice:
                    current_dice = DiceLibraryEntry(
                        die.findtext("name"), 
                        int(die.findtext("sides")),
                        die.findtext("icon")
                        )
                    self[current_dice.label] = current_dice

    def add_dice(self, dice):
        """ 
        Adds a new DiceLibraryEntry to the Custom DicePool element in 
        Config.xml.
        """
        tree = ET.parse(Resources.finder.find_resource_file("Config.xml"))
        if (float(tree.getroot().get("version")) == 1):
            dice_pools = tree.findall("DicePool")
            for pool in dice_pools:
                if (pool.get("name") == "Custom"):
                    new_dice = ET.Element("Dice")

                    name = ET.Element("name")
                    name.text = dice.label
                    new_dice.append(name)

                    sides = ET.Element("sides")
                    sides.text = str(dice.sides)
                    new_dice.append(sides)

                    icon = ET.Element("icon")
                    icon.text = dice.image
                    new_dice.append(icon)

                    pool.append(new_dice)
                    tree.write(
                        os.path.join(
                            Resources.finder.resource_directory, 
                            "Custom.xml"
                            )
                        )
        self[dice.label] = dice
                    
#==============================================================================
class DiceLibraryEntry(object):

    def __init__(self, name, sides, image):
        self.label = name
        self.sides = sides
        self.image = None
        if (not image is None):
            self.image = self.find_image(image)

    def find_image(self, icon_name):
        # first try the installed images
        image = Resources.finder.find_image_file(icon_name)
        if (image is None):
            # if it isn't there then try as an absolute path
            if (os.path.isfile(icon_name)):
                image = icon_name
        return image



#!/usr/bin/env python
# Written by: DGC

# python imports
import unittest

# local imports
import DiceRollers
import ProbabilityDensity

#==============================================================================
class utest_ProbabilityDensity(unittest.TestCase):

    def test_one_d6(self):
        pool = DiceRollers.DicePool()
        pool.add_dice("D6")
        density = ProbabilityDensity.ProbabilityDensity(pool)
        self.assertEqual(
            density[1], 
            1 / 6.0,
            "Probability of rolling a 1 is 1/6"
            )
        self.assertEqual(density[0], 0, "Probability of rolling a 0 is 0")
    
    def test_two_d6(self):
        pool = DiceRollers.DicePool()
        pool.add_dice("D6")
        pool.add_dice("D6")
        density = ProbabilityDensity.ProbabilityDensity(pool)
        self.assertEqual(density[1], 0, "Probability of rolling a 1 is 0")
        self.assertEqual(
            density[2],
            1 / 36.0,
            "Probability of rolling a 2 is 1/36"
            )
        self.assertEqual(
            density[7],
            1 / 6.0,
            "Probability of rolling a 7 is 1/6"
            )
    
    def test_loads_of_dice(self):
        pool = DiceRollers.DicePool()
        pool.add_dice("D100")
        pool.add_dice("D100")
        pool.add_dice("D100")
        pool.add_dice("D100")
    
        density = ProbabilityDensity.ProbabilityDensity(pool)
        self.assertIs(
            density.density, 
            None,
            "Too many possibilities to calculate."
            )
    
#==============================================================================
if (__name__ == "__main__"):
    unittest.main(verbosity=2)



#!/usr/bin/env python
# Written by: DGC

# python imports
from PyQt4 import QtGui, QtWebKit

# local imports
import Resources.finder

#==============================================================================
class ViewerDock(QtGui.QDockWidget):

    def __init__(self):
        super(ViewerDock, self).__init__("PDF Viewer")
        self.setWindowIcon(
            QtGui.QIcon(Resources.finder.find_image_file("Pdf.png"))
            )
        self.pdf_view = QtWebKit.QWebView()
        self.pdf_view.settings().setAttribute(
            QtWebKit.QWebSettings.PluginsEnabled, 
            True
            )
        self.setWidget(self.pdf_view)

    def set_url(self, url):
        self.pdf_view.load(url)



#!/usr/bin/env python
# Written by: DGC

# python imports

# local imports
from setuptools import setup

with open("README", "r") as readme_file:
    readme = readme_file.read()

# not strictly a good idea as it uses revision number to identify a changeset
with open(".hg/cache/branchheads", "r") as mercurial_file:
    first_line = mercurial_file.readline()
    mercurial_revision = first_line.split()[1]

setup(
    name="rputils",
    version="0." + mercurial_revision,
    author="David Corne",
    author_email="davidcorne@gmail.com",
    url="https://bitbucket.org/davidcorne/roleplay-utility",
    license="GPLv3",
    description="An application for dice rolling and reading PDF files.",
    long_description=readme,
    packages=["rputils", "rputils.Resources"],
    keywords="rputils roleplay dice pdf random",
    install_requires=["PyQt"],
    platforms=[],
    scripts=["scripts/rputils"],
    classifiers=[
        "Environment :: Win32 (MS Windows)",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Games/Entertainment",
        "Topic :: Games/Entertainment :: Board Games",
        "Topic :: Games/Entertainment :: Role-Playing",
        "Topic :: Multimedia :: Graphics :: Presentation",
        ],
    )



#!/usr/bin/env python
# Written by: DGC

# python imports
import sys
import unittest

from PyQt4 import QtGui, QtCore, QtTest

# local imports
import qtGUI

#==============================================================================
class utest_qtGUI(unittest.TestCase):

    def setUp(self):
        app = qtGUI.MainApp()
        self.dice_selecter = app.window.dice_selecter.dice_selecter

    def test_dice_order(self):
        dice = ["D3", "D4", "D6", "D8", "D10", "D12", "D20", "D100"]
        for i, die in enumerate(dice):
            button = self.dice_selecter.layout.itemAt(i).widget()
            message = str(i) + "th widget is not a " + die
            self.assertEqual(button.toolTip(), die, message)

#==============================================================================
if (__name__ == "__main__"):
    unittest.main(verbosity=2)
